/*
	pdd.c
 
    This file is part of pdd (Palm dd).
 
    A memory imaging tool for forensic analysis of Palm OS platform devices

    Copyright (c) 2002 Joe Grand
*/

#include "pdd.h"

unsigned char	        rxbuffer[MAX_BUF_SIZE]; /* raw buffers */
static unsigned char    txbuffer[MAX_BUF_SIZE]; 
static FILE			    *file;			        /* raw image output file */
static SysPktBodyType	body;                   /* parsed packet */
SlkPktHeaderType	    header;                 
SlkPktFooterType	    footer;

void wrapup (void) /* handler for ctrl-c */
{
	if (file) fclose(file);

	close_serial_port();
	fprintf(stderr, "\npdd aborted. Exiting.\n");
    exit(0);
}


void pdd (char* port, char* of, unsigned short cardNo, MemoryType memtype)
{
    unsigned int	ramBase, romBase;	// memory base locations
    unsigned int    romSize;
	unsigned int	crDate, romUsed, ramSize, freeBytes; // MemCardInfo
	unsigned char	cardName[32], manufName[32];
	unsigned short	version;
	unsigned int	count;          // ReadMemory
	unsigned int	size, base;
	unsigned char	romToken[16];   // HwrGetROMToken
	unsigned int	pType, osType;  // FtrGet
	unsigned char	ptypeStr[32], osTypeStr[8];
	unsigned int	crTime;
	struct   tm		*crt;
	unsigned char*	pTime;

	if (open_serial_port(port, DEF_TIMEOUT)) // Palm OS console mode defaults to 57.6k
	{
		fprintf(stderr, "\nError opening %s\n", port);
		close_serial_port();
		exit(1);
	}

	fprintf(stderr, "\nEnter console debug mode [<shortcut> .. 2]\n");

	pdd_MemCardInfo (cardNo, cardName, manufName, &version, &crDate, &romUsed, &ramSize, &freeBytes);

	// Now that connection has been established, 
    // reconfigure serial port for fast receive
	close_serial_port();
	open_serial_port(port, FAST_TIMEOUT);

	fprintf(stderr, "\npdd process beginning.\n");

    // display device information and write to file
    if ((file = fopen("pdd.txt", "a+")) == NULL) // create file for output, append if it exists
	{
		fprintf(stderr, "\nUnable to create pdd.txt\n");			
		exit(1);
	}

	time(&crTime);
	crt = gmtime(&crTime);
	pTime = asctime(crt);
	pTime[24] = '\0';
	fprintf(file, "Current Time: %s UCT", pTime);
	fprintf(file, "\nCard Number: %d", cardNo);
	fprintf(file, "\nCard Name: %s", cardName);
	fprintf(file, "\nManufacturer: %s", manufName);
	fprintf(file, "\nCard Version: %.4d", version);

    if (crDate) // Early OS (< 3.0) return 0x0 for creation time
    {
	    crTime = crDate - 0x7C25BE85; // Palm OS stores time in seconds from Jan. 1, 1904
		crt = gmtime(&crTime);
        pTime = asctime(crt);
        pTime[24] = '\0';
        strcat(pTime, " UCT");
    }
    else
        strcpy(pTime, "Not available");

	fprintf(file, "\nCreation Date: %s", pTime);
    	
    pdd_FtrGet (sysFtrCreator, sysFtrNumROMVersion, &osType);
    // Convert version number into string
    // 0xMMmfsbbb, where MM is major version, m is minor version
    // f is bug fix, s is stage: 3-release,2-beta,1-alpha,0-development,
    // bbb is build number for non-releases 
    // V1.12b3   would be: 0x01122003
    // V2.00a2   would be: 0x02001002
    // V1.01     would be: 0x01013000
    sprintf(osTypeStr, "%X.%X.%X", (osType >> 24) & 0xff, (osType >> 20) & 0xf, (osType >> 16) & 0xf);

	// Get processor type (68328, 'EZ, or 'VZ)
    // 0xMMMMRRRR, where MMMM is the processor model and RRRR is the revision.
	// Configure memory base locations and set ROM size    
    
    /* SysGetMemCardInfo returns the space actually used by Palm OS, NOT the actual ROM size.
     * We can fix this by reading the DragonBall Chip Select registers, since they are memory-mapped.
     * Assume ROM is located at CSA0. If size cannot be determined, fall back to romSize returned
     * by SysGetMemCardInfo.
     */
    pdd_FtrGet (sysFtrCreator, sysFtrNumProcessorID, &pType);
	switch (pType & sysFtrNumProcessorMask)
	{
		case sysFtrNumProcessor328: 
			ramBase = hwrCardBase + (cardNo * hwrCardSize);
			romBase = hwrROMBase;
			strcpy(ptypeStr, "Motorola DragonBall 68328");

            pdd_ReadMemory (hwrcsASelect0, 4);
            // Calculate ROM size based on Address Mask
            switch(body.data[2])
            {
                case hwrChipSelectSize64k:
                    romSize=0x10000;
                    break;
                case hwrChipSelectSize128k:
                    romSize=0x20000;
                    break;
                case hwrChipSelectSize256k:
                    romSize=0x40000;
                    break;
                case hwrChipSelectSize512k:
                    romSize=0x80000;
                    break;
                case hwrChipSelectSize1M:
                    romSize=0x100000;
                    break;
                case hwrChipSelectSize2M:
                    romSize=0x200000;
                    break;
                case hwrChipSelectSize4M:
                    romSize=0x400000;
                    break;                    
                case hwrChipSelectSize8M:
                    romSize=0x800000;
                    break;
                case hwrChipSelectSize16M:
                    romSize=0x1000000;
                    break;
                default:
                    romSize=romUsed;
                    break;
            }
			break;

		case sysFtrNumProcessorEZ: 	
			ramBase = hwrCardBaseEZ + (cardNo * hwrCardSizeEZ);
			romBase = hwrROMBaseEZ;
			strcpy(ptypeStr, "Motorola DragonBall 68EZ328");
            pdd_ReadMemory (hwrEZcsASelect, 2);
            switch (body.data[1] & hwrEZChipSelectSizeMask)
            {
                case hwrEZChipSelectSize128k:
                    romSize=0x20000;
                    break;
                case hwrEZChipSelectSize256k:
                    romSize=0x40000;
                    break;
                case hwrEZChipSelectSize512k:
                    romSize=0x80000;
                    break;
                case hwrEZChipSelectSize1M:
                    romSize=0x100000;
                    break;
                case hwrEZChipSelectSize2M:
                    romSize=0x200000;
                    break;
                case hwrEZChipSelectSize4M:
                    romSize=0x400000;
                    break;                    
                case hwrEZChipSelectSize8M:
                    romSize=0x800000;
                    break;
                case hwrEZChipSelectSize16M:
                    romSize=0x1000000;
                    break;
                default:
                    romSize=romUsed;
                    break;                    
            }
			break;

		case sysFtrNumProcessorVZ:  
			ramBase = hwrCardBaseVZ + (cardNo * hwrCardSizeVZ);
			romBase = hwrROMBaseVZ;
			strcpy(ptypeStr, "Motorola DragonBall 68VZ328");
            pdd_ReadMemory (hwrVZcsASelect, 2);
            switch (body.data[1] & hwrVZChipSelectSizeMask)
            {
                case hwrVZChipSelectSize128k:
                    romSize=0x20000;
                    break;
                case hwrVZChipSelectSize256k:
                    romSize=0x40000;
                    break;
                case hwrVZChipSelectSize512k:
                    romSize=0x80000;
                    break;
                case hwrVZChipSelectSize1M:
                    romSize=0x100000;
                    break;
                case hwrVZChipSelectSize2M:
                    romSize=0x200000;
                    break;
                case hwrVZChipSelectSize4M:
                    romSize=0x400000;
                    break;                    
                case hwrVZChipSelectSize8M:
                    romSize=0x800000;
                    break;
                case hwrVZChipSelectSize16M:
                    romSize=0x1000000;
                    break;
                default:
                    romSize=romUsed;
                    break;
            }
			break;

		default:
			fprintf(stderr, "\nCannot identify processor type.\n\n");
			close_serial_port();
			exit(1);
			break;
	}

    fprintf(file, "\nPalm OS Version: %s", osTypeStr);
   	fprintf(file, "\nProcessor Type: %s", ptypeStr); 
	fprintf(file, "\nRAM Size: %d bytes", ramSize);
	fprintf(file, "\nFree RAM: %d bytes", freeBytes);
    fprintf(file, "\nROM Size: %d bytes", romSize);
    fprintf(file, "\nROM Used By OS: %d bytes", romUsed);

    if (osType >= 0x03000000) 
    {
        // Get Flash ROM ID (applicable only for devices with Flash ROM)
        pdd_HwrGetROMToken (cardNo, sysROMTokenSnum, romToken);
    }
    else
        strcpy(romToken, "Not available");

	fprintf(file, "\nFlash ID: %s", romToken);
    fprintf(file, "\nImage Output File: ");
    if (*of) fprintf(file, "%s", of);
    else fprintf(file, "Standard output");
    
    fprintf(file, "\nImage Memory Type: ");

	// configure variables for Read Memory loop
    if (cardNo > 0) memtype = RAM; // cards > 0 treated as RAM?
	switch (memtype)
	{
		case RAM:
			base = ramBase;
			size = ramSize;
            fprintf(file, "RAM");
			break;

		case ROM:
			base = romBase;
			size = romSize; 
            fprintf(file, "ROM");
			break;
	}
    fprintf(file, "\nStarting Address: $%.8X\n\n", base);
	if (file) fclose(file); // end of device information, close file
    
    if (*of)
    {
        if ((file = fopen(of, "wb")) == NULL) // create file for output
	    {
		    fprintf(stderr, "\nUnable to create %s\n", of);			
		    exit(1);
	    }
    }
    else
    {
        file = stdout;

        // Set stdout to have binary mode, since Windows defaults to a text translation mode
        if (_setmode(_fileno(stdout), _O_BINARY) == -1)
        {
		    fprintf(stderr, "\nUnable to set stdout mode to binary\n");			
		    exit(1);
        }
    }

	for (count = 0; count < size; count += sysPktMaxMemChunk)
	{
		pdd_ReadMemory (count + base, sysPktMaxMemChunk); // retrieved data stored in body global
		if (header.bodySize == sysPktMaxMemChunk + 2) // if the correct amount of data was receievd
		{
             // if no file specified, data should go to standard output
			if (file) 
                fwrite(body.data, 1, sysPktMaxMemChunk, file);
		}
		else
		{
			fprintf(stderr, "\nRead Memory command failed.\n");
			exit(1);
		}
	}
	fprintf(stderr, "\n"); // extra newline for consistancy

	if (file) fclose(file);

	// Call sysTrapSysReset to soft-reset the device. This will disable the
	// Palm OS debugger and return it to a known state
	pdd_SysReset();

	close_serial_port();
	fprintf(stderr, "\npdd successful. Exiting.\n");
}


void pdd_ReadMemory (unsigned int address, unsigned short numBytes)
{
	int	plen;	// packet length 
	struct	SysPktReadMemCmdType	CMD[1];

	CMD[0].command = sysPktReadMemCmd;
	CMD[0].address = address;			// Address to read
	bswap32(CMD[0].address);
	CMD[0].numBytes = numBytes;			// # of bytes to read
	bswap16(CMD[0].numBytes);

	plen = create_packet(slkSocketConsole, slkSocketConsole, slkPktTypeSystem, 0x04, 
						 CMD, sizeof(SysPktReadMemCmdType));

	while (body.command != sysPktReadMemRsp) // wait for response from Palm Console
	{
		write_buffer(txbuffer, plen);
		flush_buffer (rxbuffer);
		read_buffer(rxbuffer, MAX_BUF_SIZE);
		parse_packet(rxbuffer); // populate header, body, and footer
	};
    if (verify_checksum())
	{
		fprintf(stderr, "\nCRC-16 Checksum invalid.\n");
		exit(1);
	}
}


void pdd_SysReset (void)
{
	int	plen;	// packet length
	struct 	SysPktRPCType	RPC[1];

	RPC[0].command = sysPktRPCCmd;
	RPC[0].trapWord = sysTrapSysReset;
	bswap16(RPC[0].trapWord);
	RPC[0].numParams = 0;

	plen = create_packet(slkSocketConsole, slkSocketConsole, slkPktTypeSystem, 0x89, 
						 RPC, sizeof(SysPktRPCType) - sizeof(SysPktRPCParamType));

	fprintf(stderr, "\nResetting Palm OS device.\n");

	// send multiple times to guarantee reset
	write_buffer(txbuffer, plen);
	write_buffer(txbuffer, plen);
	write_buffer(txbuffer, plen);
}


void pdd_FtrGet (unsigned int creator, unsigned short featureNum, unsigned int *valueP)
{
	int	plen;	// packet length
	struct 	SysPktRPCType		RPC[32];	// allocate extra memory since parameter sizes vary
	struct 	SysPktRPCParamInfo	*rpc;

	RPC[0].command = sysPktRPCCmd;
	RPC[0].trapWord = sysTrapFtrGet;
	bswap16(RPC[0].trapWord);
	RPC[0].numParams = 3;
	bswap16(RPC[0].numParams);
	// &valueP
	RPC[0].param->byRef = 1;
	RPC[0].param->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param + RPC[0].param->size + 2); // point to next param struct
	// featureNum
	rpc->byRef = 0;
	rpc->size = sizeof(unsigned short);
	bswap16(featureNum);
	rpc->data[0] = featureNum;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// creator
	rpc->byRef = 0;
	rpc->size = sizeof(unsigned int);
	bswap32(creator);
	rpc->data[0] = creator;
	rpc->data[1] = creator >> 16;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct

	plen = create_packet(slkSocketConsole, slkSocketConsole, slkPktTypeSystem, 0x02, 
						 RPC, (unsigned short)((unsigned short)rpc - (unsigned short)RPC));

	while (body.command != sysPktRPCRsp) // wait for response from Palm Console
	{
		write_buffer(txbuffer, plen);
		flush_buffer (rxbuffer);
		read_buffer(rxbuffer, MAX_BUF_SIZE);
		parse_packet(rxbuffer); // populate header, body, and footer
	};
    if (verify_checksum())
	{
		fprintf(stderr, "\nCRC-16 Checksum invalid.\n");
		exit(1);
	}

	// Parse RPC packet
	memmove (RPC, (struct SysPktRPCType*)&body, header.bodySize);
	bswap16(RPC[0].trapWord);
	bswap16(RPC[0].numParams);
	bswap32(RPC[0].resultA0);
	bswap32(RPC[0].resultD0);

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param); // point to next param struct
	bswap16(rpc->data[0]);
	*valueP = rpc->data[0];
	*valueP <<= 16;
	bswap16(rpc->data[1]);
	*valueP |= rpc->data[1];
}


void pdd_HwrGetROMToken (unsigned short cardNo, unsigned int token, unsigned char *data)
{
	int	plen;	// packet length
	struct 	SysPktRPCType		RPC[32];	// allocate extra memory since parameter sizes vary
	struct 	SysPktRPCParamInfo	*rpc;
	
	unsigned int dataP;
	unsigned short size;

	RPC[0].command = sysPktRPCCmd;
	RPC[0].trapWord = sysTrapHwrGetROMToken;
	bswap16(RPC[0].trapWord);
	RPC[0].numParams = 4;
	bswap16(RPC[0].numParams);
	// &size
	RPC[0].param->byRef = 1;
	RPC[0].param->size = sizeof(unsigned short);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param + RPC[0].param->size + 2); // point to next param struct
	// &dataP
	rpc->byRef = 1;
	rpc->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// token
	rpc->byRef = 0;
	rpc->size = sizeof(unsigned int);
	bswap32(token);
	rpc->data[0] = token;
	rpc->data[1] = token >> 16;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// cardNo
	rpc->byRef = 0;
	rpc->size = sizeof(unsigned short);
	bswap16(cardNo);
	rpc->data[0] = cardNo;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct

	plen = create_packet(slkSocketConsole, slkSocketConsole, slkPktTypeSystem, 0x03, 
						 RPC, (unsigned short)((unsigned short)rpc - (unsigned short)RPC));

	while (body.command != sysPktRPCRsp) // wait for response from Palm Console
	{
		write_buffer(txbuffer, plen);
		flush_buffer (rxbuffer);
		read_buffer(rxbuffer, MAX_BUF_SIZE);
		parse_packet(rxbuffer); // populate header, body, and footer
	};
    if (verify_checksum())
	{
		fprintf(stderr, "\nCRC-16 Checksum invalid.\n");
		exit(1);
	}

	// Parse RPC packet
	memmove (RPC, (struct SysPktRPCType*)&body, header.bodySize);
	bswap16(RPC[0].trapWord);
	bswap16(RPC[0].numParams);
	bswap32(RPC[0].resultA0);
	bswap32(RPC[0].resultD0);

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param); // point to next param struct	bswap16(rpc->data[0]);
	bswap16(rpc->data[0]);
	size = rpc->data[0];

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	bswap16(rpc->data[0]);
	dataP = rpc->data[0];
	dataP <<= 16;
	bswap16(rpc->data[1]);
	dataP |= rpc->data[1];

	if ((dataP == 0x00000000) || (size == 0))
	{
		strcpy(data, "Not available");
		return;
	}
	
	pdd_ReadMemory (dataP, size);
	if (body.data[0] == 0xFF)
	{
		strcpy(data, "Not available");
		return;
	}
	else // We have a valid serial numer, so calculate checksum
	{
		unsigned short count;
		unsigned char checkSum = 0, ckStr[3];

		strncpy(data, body.data, size);
		data[size] = '\0';

		// Calculate the checksum
		// Start with 0, add each digit, 
		// then rotate result one bit to the left and repeat.
		for (count = 0; count < size; count++)
		{
			checkSum += data[count];
			checkSum = checkSum = (checkSum<<1) | ((checkSum & 0x80) >> 7);
		}

		// Add the two hex digits (nibbles) together, +2
		// (range: 2-31 ==> 2-9, A-W)
		// By adding 2 to the result before converting to ASCII,
		// we eliminate the numbers 0 and 1, which can be
		// difficult to distinguish from the letters O and I.
		checkSum = ((checkSum>>4) & 0x0F) + (checkSum & 0x0F) + 2;

		// Draw the dash and the checksum digit right after it
		ckStr[0] = '-';
		ckStr[1] = ((checkSum < 10) ? (checkSum +'0'):(checkSum -10 +'A'));
		ckStr[2] = '\0';
		strcat(data, ckStr);
	}	 
}


void pdd_MemCardInfo (unsigned short cardNo, unsigned char *cardName, unsigned char *manufName, 
					  unsigned short *version, unsigned int *crDate,  unsigned int *romSize, 
					  unsigned int *ramSize, unsigned int *freeBytes)
{
	int	plen;	// packet length
	struct 	SysPktRPCType		RPC[32];	// allocate extra memory since parameter sizes vary
	struct 	SysPktRPCParamInfo	*rpc;

	RPC[0].command = sysPktRPCCmd;
	RPC[0].trapWord = sysTrapMemCardInfo;
	bswap16(RPC[0].trapWord);
	RPC[0].numParams = 8;
	bswap16(RPC[0].numParams);
	// &freeBytes
	RPC[0].param->byRef = 1;
	RPC[0].param->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param + RPC[0].param->size + 2); // point to next param struct
	// &ramSize
	rpc->byRef = 1;
	rpc->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// &romSize
	rpc->byRef = 1;
	rpc->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// &crDate
	rpc->byRef = 1;
	rpc->size = sizeof(unsigned int);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// &version
	rpc->byRef = 1;
	rpc->size = sizeof(unsigned short);
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// &manufName
	rpc->byRef = 1;
	rpc->size = 32;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// &cardName
	rpc->byRef = 1;
	rpc->size = 32;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	// cardNo
	rpc->byRef = 0;
	rpc->size = sizeof(unsigned short);
	bswap16(cardNo);
	rpc->data[0] = cardNo;
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct

	plen = create_packet(slkSocketConsole, slkSocketConsole, slkPktTypeSystem, 0x01, 
						 RPC, (unsigned short)((unsigned short)rpc - (unsigned short)RPC));

	while (body.command != sysPktRPCRsp) // wait for response from Palm Console
	{
		write_buffer(txbuffer, plen);
		flush_buffer (rxbuffer);
		read_buffer(rxbuffer, MAX_BUF_SIZE);
		parse_packet(rxbuffer); // populate header, body, and footer
	};
    if (verify_checksum())
	{
		fprintf(stderr, "\nCRC-16 Checksum invalid.\n");
		exit(1);
	}

	// Parse RPC packet
	memmove (RPC, (struct SysPktRPCType*)&body, header.bodySize);
	bswap16(RPC[0].trapWord);
	bswap16(RPC[0].numParams);
	bswap32(RPC[0].resultA0);
	bswap32(RPC[0].resultD0);

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) RPC[0].param); // point to next param struct
	bswap16(rpc->data[0]);
	*freeBytes = rpc->data[0];
	*freeBytes <<= 16;
	bswap16(rpc->data[1]);
	*freeBytes |= rpc->data[1];

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	bswap16(rpc->data[0]);
	*ramSize = rpc->data[0];
	*ramSize <<= 16;
	bswap16(rpc->data[1]);
	*ramSize |= rpc->data[1];

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	bswap16(rpc->data[0]);
	*romSize = rpc->data[0];
	*romSize <<= 16;
	bswap16(rpc->data[1]);
	*romSize |= rpc->data[1];

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	bswap16(rpc->data[0]);
	*crDate = rpc->data[0];
	*crDate <<= 16;
	bswap16(rpc->data[1]);
	*crDate |= rpc->data[1];

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	bswap16(rpc->data[0]);
	*version = rpc->data[0];
	
	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	strncpy(manufName, (unsigned char*)rpc->data, 32);

	rpc = (struct SysPktRPCParamInfo*) ((unsigned char*) rpc + rpc->size + 2); // point to next param struct
	strncpy(cardName, (unsigned char*)rpc->data, 32);
}


/* Create Debugger Protocol packet */ 
/* defined in Debugger Protocol Reference (Palm OS Programming Development Tools Guide */
int create_packet (unsigned char dest, unsigned char src, 
				   unsigned char type, unsigned char transId,
				   void* dataP, unsigned short bodySize)
{
	int	plen;	// packet length

	header.signature1	= slkPktHeaderSignature1;
	header.signature2	= slkPktHeaderSignature2;
	header.dest			= dest;
	header.src			= src;
	header.type			= type;
	header.bodySize		= bodySize;
	header.transId		= transId;
	header.checksum		= calc_hdr_checksum (0, (char*)&header.signature1, ((char*)&header.checksum) - ((char*)&header.signature1));
	bswap16(header.signature1);
	bswap16(header.bodySize);

	memcpy (&body, dataP, bodySize);

	// Calculate the footer checksum
	footer.crc16 = calc_crc16 (&header, sizeof(header), 0);
	footer.crc16 = calc_crc16 (&body, bodySize, footer.crc16);
	bswap16(footer.crc16);

	// Copy packet into buffer for serial transmission
	plen = sizeof(header);
	memcpy(txbuffer, &header, plen);
	memcpy(&txbuffer[plen], &body, bodySize);
	plen += bodySize;
	memcpy(&txbuffer[plen], &footer, sizeof(footer));
	plen += sizeof(footer);

	return (plen);
}


void parse_packet (char* lpBuf)
{
	int s = sizeof(SlkPktHeaderType);

	memcpy(&header, lpBuf, s);
	bswap16(header.signature1);
	bswap16(header.bodySize);
	
	memcpy(&body, lpBuf+s, header.bodySize);
	s += header.bodySize;
	
	memcpy(&footer, lpBuf+s, sizeof(SlkPktFooterType));
	bswap16(footer.crc16);
}
